package com.lucasmufato.bd2.examples;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Purpose: contiene distintas formas de iterar lista.
 *
 * @author : lucasmufato
 **/
public class IterationExamples {

  public static void main(String[] args){
    List<String> lista = new ArrayList<>();
    lista.add("primero");
    String[] array = new String[]{"segundo", "tercero", "cuarto"};
    lista.addAll( Arrays.asList(array) );

    forComun(lista);
    forEach(lista);
    iterable(lista);
    anonimo(lista);
    anonimoLambda(lista);
    stream(lista);
  }
  private static void forComun(List<String> lista) {
    System.out.println("For Comun");
    for(int i=0; i<lista.size(); i++){
      System.out.println(lista.get(i));
    }
    System.out.println();
  }

  /**
   *  Se aplica sobre un iterable algo que implemente iterable
   *  el iterable esta desde java 1.5
   * @param lista
   */
  private static void forEach(List<String> lista) {
    System.out.println("For Each");
    for( String elemento : lista){
      System.out.println(elemento);
    }
    System.out.println();
  }

  /**
   * Se aplica sobre un iterador.
   * desde java 1.2
   *
   * @param lista
   */
  private static void iterable(List<String> lista) {
    System.out.println("iterable");
    Iterator<String> iterator = lista.iterator();
    while ( iterator.hasNext() ){
      System.out.println( iterator.next() );
    }
    System.out.println();
  }

  /**
   * desde java 1.8 se aplica sobre un iterable.
   * Se llamafuncion anonima por que la misma no tiene nombre, no puede ser llamada desde otro lugar
   * unas ves que se ejecuta 'muere'
   *
   * @param lista
   */
  private static void anonimo(List<String> lista) {
    System.out.println("anonimo");
    lista.forEach( elemento -> {
      System.out.println(elemento);
    });
    System.out.println();
  }

  /**
   * Desde java 1.8 se agregan caracteristica funcionales al lenguaje, por lo que se pueden pasar
   * metodos y funciones lambda.
   *
   * @param lista
   */
  private static void anonimoLambda(List<String> lista) {
    System.out.println("anonimo Lambda");
    lista.forEach(System.out::println);
    System.out.println();
  }

  /**
   * en java 1.8 se agregan Streams. son como flujos de datos para realizar operaciones sobre
   * los mismos. sumado a funciones lambda facilitan la programacion y lectura del codigo.
   *
   * @param lista
   */
  private static void stream(List<String> lista) {
    System.out.println("stream");
    lista.stream()
        .peek(e -> System.out.println(e))
        .collect(Collectors.toList());
    System.out.println("");
  }

}
