package com.lucasmufato.bd2.repository;

import com.lucasmufato.bd2.model.Employee;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

/**
 * Purpose: persist Employee object in the database
 *
 * @author : lucasmufato
 **/
public class EmployeeDaoImplementation implements EmployeeDao {

  private Connection connection;

  public EmployeeDaoImplementation(Connection conection) {
    this.connection = conection;
  }

  /**
   * inserto un employee usando statement comun.
   *
   * @param employee
   */
  @Override
  public void insert(Employee employee) {
    String query = "INSERT INTO employee(id, salary, name) VALUES (" +
        employee.getId() + ", " +
        employee.getSalary() + ", '" +
        employee.getName() +
        "')";
    try {
      Statement statement = this.connection.createStatement();
      statement.execute(query);
    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }

  /**
   * obtengo un employee buscando x su id con prepared statement.
   * @param queryId
   * @return
   */
  @Override
  public Employee getById(int queryId) {
    String query = "SELECT * FROM employee WHERE ID = ?";
    try {
      PreparedStatement preparedStatement = this.connection.prepareStatement(query);
      preparedStatement.setInt(1, queryId);
      ResultSet resultSet = preparedStatement.executeQuery();

      if( resultSet.next() ){
        Employee e = new Employee();

        int id = resultSet.getInt("id");
        e.setId(id);

        String name = resultSet.getString("name");
        e.setName(name);

        long salary = resultSet.getLong("salary");
        e.setSalary(salary);

        return e;
      }
      return null;

    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }

  /**
   * Se ejecuta un update usando prepare statmente y pasando los parametros por posicion.
   * @param employee
   * @return
   */
  @Override
  public Employee merge(Employee employee) {
    String query = "UPDATE employee SET ID = ?, NAME = ?, SALARY = ? WHERE ID = ?";
    try {
      PreparedStatement preparedStatement = this.connection.prepareStatement(query);

      preparedStatement.setInt(1, employee.getId());
      preparedStatement.setString(2, employee.getName());
      preparedStatement.setLong(3, employee.getSalary());
      preparedStatement.setInt(4, employee.getId());

      preparedStatement.execute();

      return employee;
    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public void remove(Employee employee) {
    String query = "DELETE FROM employee WHERE ID =  " + employee.getId();
    try {
      Statement statement = this.connection.createStatement();
      statement.execute(query);
    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public void removeAll() {
    String query = "DELETE FROM employee";
    try {
      Statement statement = this.connection.createStatement();
      statement.execute(query);
    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public List<Employee> findAll() {
    List<Employee> employees = new ArrayList<>();
    String query = "SELECT * FROM employee";
    try {
      Statement statement = this.connection.createStatement();
      ResultSet resultSet = statement.executeQuery(query);
      while( resultSet.next() ){
        Employee e = new Employee();

        int id = resultSet.getInt("id");
        e.setId(id);

        String name = resultSet.getString("name");
        e.setName(name);

        long salary = resultSet.getLong("salary");
        e.setSalary(salary);

        employees.add(e);
      }
      return employees;

    } catch (SQLException e) {
      throw new RuntimeException(e);
    }

  }

  @Override
  public List<Employee> findAllSalaryGreaterThan(long querySalary) {
    List<Employee> employees = new ArrayList<>();
    String query = "SELECT * FROM employee e WHERE e.salary > "+querySalary+"";
    try {
      Statement statement = this.connection.createStatement();
      ResultSet resultSet = statement.executeQuery(query);
      while( resultSet.next() ){
        Employee e = new Employee();

        int id = resultSet.getInt("id");
        e.setId(id);

        String name = resultSet.getString("name");
        e.setName(name);

        long salary = resultSet.getLong("salary");
        e.setSalary(salary);

        employees.add(e);
      }
      return employees;

    } catch (SQLException e) {
      throw new RuntimeException(e);
    }
  }
}
